C========================================================================
C
C     smeedc.f    --- for non-centrosymmetric samples
C
C     MAXIMUM ENTROPY ELECTRON DENSITY ANALYSIS PROGRAM
C     FOR X-RAY POWDER DIFFRACTION DATA
C     REFERENCE : J.APPL.CRYST. (1990) 23, 526-534
C     COPYRIGHT BY SHINTARO KUMAZAWA ( NAGOYA UNIVERCITY )
C     ALL RIGHT RESERVED.
C
C     - UPDATE LABEL -
C
C     VERSION 1.00  1990-04-24  BY SHINTARO KUMAZAWA
C      ORIGINAL PROGRAM, MEMTARO.FORT.
C     VERSION 1.10  1990-11-08  BY YOSHIKI KUBOTA
C      ADD PREFERED ORIENTATION CORRECTION VERSION.
C     VERSION 2.00  1991-10-16  BY SHINTARO KUMAZAWA
C      START MEED SYSTEM PROJECT.
C      CHANGE SYMMETRY TREATMENT AND SOME CALCULATION ALGORITHM.
C     VERSION 2.12  1991-10-25  BY SHINTARO.KUMAZAWA
C      BUG FIX & OPTIMIZE SOME CALCULATION ALGORITHM.
C     VERSION 3.00  1991-11-15  BY SHINTARO KUMAZAWA
C      MODIFIED BY USING OF MAU METHOD.
C     Data input now read free-format. Better handling of divergence errors 
C      - 12.1995 k.b.
C     Data output format changed  - K. Burger 3.1996
C     MaxEnt-routine optimized for velocity.
C      New COMMON variables: rhelp1,2,4,5,6(MREF) for MAXENT-SUBROUTINE.
C      Changes within loop over maxent cycles.
C      Memorize FCAL and GCAL from last successful cycle in case of
C      divergence - K. Burger 3.96 
C     ramda is varied automatically: in case of divergence ramda=ramda*0.5
C      if cycle was successful: ramda is increased a bit.
C      Bug in MKASYM routine fixed: wrong results in the case of
C      different pixel resolution: NPIX(1)<>NPIX(2 or 3).
C      Translation vectors ITRS and ISUB now real numbers: RTRS and RSUB.
C      Program split into modules; no COMMON blocks any more. (k.b. 4.1996)
C
C     - CAUTION !! -
C
C     (1) IF YOU TRY A SAMPLE HAVING NO CENTRE OF SYMMETRY, YOU MUST
C         REMOVE THE COMMENT MARK, 'CENT'.
C     (2) YOU MAY NOT USE AN OLD FORMAT MEM DATA SET.
C     (3) SUBROUTINES DATE & WATCH CAN BE ONLY USED BY FACOM COMPUTER.
C     (4) PARAMETERS HAVE TO BE CHANGED IN THE SUBROUTINES, TOO.
C
C     - UNIT NUMBER OF DATA FILES -
C
C     10 : MEED DATA SET ( THIS IS POWLS6.EXE COMPATIBLE FORMAT. )
C     20 : DENSITY DATA SET ( THIS IS READABLE BY MECONT.EXE. )
C     30 : ANALYSIS DOCUMENT DATA SET
C     40 : SPACE GROUP DATA BASE 'SPGRA', OFFERED BY FUJIO IZUMI.
C
C====================================================================
C
C
C*** MAIN ROUTINE ***
C
      PROGRAM smeedc
C
      IMPLICIT INTEGER (I-N)
      IMPLICIT REAL*8 (A-H,O-Z)
C       MREF = max. number of reflections
C       MASYM = max. number of pixels in asymmetric unit of cell
C       MPIX = max. number of pixels along one cell axis (resolution)
C       MHIST = max. number of changes in ramda-parameter done by the user
C       ALPHA = Final value of constraint
C
      IMPLICIT none
C
      integer*4 nasym,masym,ia
      integer mhist,mref,mpix
      real*8 alpha
C
      PARAMETER ( mref=70,masym=13000,mpix=65)
      PARAMETER ( mhist=2 )
      PARAMETER ( alpha=1.0 )
C
C     *** variables connected with the scattering-density values in the 
C     asymmetric unit of the cell:
      real*8 RDEO(masym),RDEN(masym),RASYM(masym) 
C
C     *** variables connected with the coordinates of the pixels:
      integer NPIX(3),IASYM(3,masym)
C
C     *** variables connected with the structure factors given as input:
C     *** FOBS: unique reflections, GOBS: groups of non-unique refl.
      real*8 FOBS(2,mref),FCAL(2,mref),GCAL(mref/2),
     $    SIGF(mref),PROF(mref),GOBS(mref/2),SIGG(mref/2)
      real*8 tnref
      integer IDX(3,mref),NGRP(mref/2),nref,nref1,nref2
C
C     *** cell-constants and others:
      real*8 CELL(6),pop,rtasym
      integer jph,jpk,jpl
      real*8 pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,totale,ff1,ff2
C
C     *** variables connected with the entropy-functions:
      real*8 rfact,wrfact,ents,entj,qfunc,conf,cong,connew,
     $  delta,ramda
C
C     *** arrays to memorize lquantities of last successful cycle 
      real*8 FCALO(2,mref),GCALO(mref),confold,deltaold,rfacold,
     $  wrfacold,conold,congold
C
C     *** variables connected with symmetry operators:
      integer IROT(3,3,48),nsym,ncen,nsub
      real*8 RTRS(3,48),RSUB(3,4)
C
C     *** tables of cos- and sin-values for each reflection and pixel:
      real*8 RMIC(mref,masym)
      real*8 RMIS(mref,masym)
C
C     *** variables connected with ramda-history list:
      integer IHIST(MHIST),nhist
      real*8 FHIST(2,MHIST)
C
C     *** variables connected with multiplicities of reflections given as groups
C     of overlayd reflections (G_obs, G-constraints):
      real*8 PMUL(mref/2),PMULS(mref/2)
C
C     *** auxilary arrays for maxent subroutine
      real*8 RHELP1(mref),RHELP2(mref),RHELP4(mref),
     $   RHELP5(mref),RHELP6(mref)
C
C     *** auxilary array for mkasym subroutine
      integer IDE(0:mpix,0:mpix,0:mpix)
C
C     *** other variables:
      CHARACTER title*200,filmem*50,filden*50,fildoc*50,
     $          aspg*10
      character*10 APRIOR(2)
      character*5 AGSW(2)
      integer iprior,igsw,nyes,iptyp,jph,jpk,jpl,nsg,nset,
     $  ierr,nhist,itcycl,ngsw,igsw,iref,nn,i,j,
     $  ncycl,iii,icycl,ibadruns,igoodcyc
C     integer ITODAY(4)
      DATA (APRIOR(J),J=1,2)/'FLAT.     ','FILE DATA. '/
      DATA (AGSW(J),J=1,2)/'F & G','  F  '/
C
C
 1090 format(2(3(I3,1X),G18.13,1x))  ! for display of asym. unit pixels
 2010 FORMAT(BN,A50)
 2020 FORMAT(BN,I5)
 3000 FORMAT(4(/' '))
 3010 FORMAT(///'   MMM            MMM   EEEEEEEEEEEEEEE   EEEEEEEEEE',
     $       'EEEEE   DDDDDDDDDD'/'   MMMMMM      MMMMMM   EEEEEEEEEE',
     $       'EEEEE   EEEEEEEEEEEEEEE   DDDDDDDDDDDD'/'   MMM MMM    ',
     $       'MMM MMM   EEE               EEE               DDD      ',
     $       '  DDD'/'   MMM  MMM  MMM  MMM   EEE               EEE  ',
     $       '             DDD         DDD'/'   MMM   MMMMMM   MMM   ',
     $       'EEEEEEEEEEEEEEE   EEEEEEEEEEEEEEE   DDD          DDD'
     $       /'   MMM     MM     MMM   EEEEEEEEEEEEEEE   EEEEEEEEEEEE',
     $       'EEE   DDD          DDD'/'   MMM            MMM   EEE   ',
     $       '            EEE               DDD         DDD'/'   MMM ',
     $       '           MMM   EEE               EEE               DD',
     $       'D        DDD'/'   MMM            MMM   EEEEEEEEEEEEEEE ',
     $       '  EEEEEEEEEEEEEEE   DDDDDDDDDDDD'/'   MMM            MM',
     $       'M   EEEEEEEEEEEEEEE   EEEEEEEEEEEEEEE   DDDDDDDDDD'
     $       ///'   MAXENT ELECTRON DENSITY ANALYSIS PROGRAM FOR X-RA',
     $       'Y POWDER DIFFRACTION DATA'//'   REFERENCE : J.APPL.CRYS',
     $       'T. (1990) 23, 526-534'//'   COPYRIGHT BY SHINTARO KUMAZ',
     $       'AWA (C) 1991,   ALL RIGHT RESERVED.')
 3020 FORMAT(///' *********   INITIAL STATE OF MEED ANALYSIS   ******',
     $       '*************************'//' TITLE : ',A200
     $       //' MEED DATA FILE NAME     = ',A50
     $        /' DENSITY DATA FILE NAME  = ',A50
     $        /' DOCUMENT DATA FILE NAME = ',A50
     $       //' CELL CONSTANT = ',5(F8.3,','),F8.3
     $        /' SPACE GROUP   = ',A10,'  ,  TOTAL CHARGE = ',F10.3
     $        /' P.O. CORRECT  = ',F10.5,'  ,  TYPE = ',I2,
     $         '  ,  VECTOR = ',2(I3,','),I3
     $        /' NUMBER OF PIXELS = ',I7,' / ',I7,'   (',2(I3,','),
     $         I3,' )'
     $       //' REFLECTIONS OF CONSTRAINT FUNCTION  F-TYPE = ',I3,
     $         ' , G-TYPE = ',I3
     $        /' INITIAL DENSITY DISTRIBUTION IS ',A
     $        /' THIS SESSION IS ANALYZED BY USING ',A,' CONSTRAINT.'
     $        /' TRIAL LAGRANGE MULTIPLIER   RAMDA = ',F10.7,'.')
 3030 FORMAT( '  con=',F18.7,' conf=',F15.4,' cong=',F15.4
     $        /' delt=',F18.7,'    R=',F15.4,'   wR=',F15.4)
 3040 FORMAT(  ' ents=',F18.7,' entj=',F15.4,' lagr=',F15.4)
 3060 FORMAT(' CYCLES ',I5)
 3130 FORMAT(/' *********   WAIT A MOMENT ! THE PHASE MATRIX ',
     $       'TABLE IS CALCULATED. ******')
 3500 FORMAT(/ ' INPUT MEED DATA SET NAME.     = ')
 3510 FORMAT(  ' INPUT DENSITY DATA SET NAME.  = ')
 3520 FORMAT(  ' INPUT DOCUMENT DATA SET NAME. = ')
 3530 FORMAT(  ' SELECT INITIAL DENSITY DATA TYPE.  (0) FLAT , (1) ',
     $       'FILE DATA   = ')
 3540 FORMAT(  ' DO YOU USE THE G-CONSTRAINT DATA IN MEM CALCULATIO',
     $       'N ?  YES(0)/NO(1)  = ')
 3550 FORMAT(  ' ARE YOU SURE ?  YES(0)/NO(1)  = ')
 3560 FORMAT(/' INPUT NCYCL. ( >0:ITERATION CYCLES , <0:CHANGE RAMD',
     $       'A , =0:EXIT LOOP )  = ')
C
C*** INPUT FILENAMES AND INITIAL CONDITIONS. ***
C
      WRITE(6,3010)
  201 WRITE(6,3500)
      READ(5,2010) FILMEM
      WRITE(6,3510)
      READ(5,2010) FILDEN
      WRITE(6,3520)
      READ(5,2010) FILDOC
      WRITE(6,3530)
      READ(5,2020) IPRIOR
      WRITE(6,3540)
      READ(5,2020) IGSW
      WRITE(6,3550)
      READ(5,2020) NYES
      IF (NYES.NE.0) GO TO 201
C
C*** READ MEED DATA AND MAKE SOME CONSTANTS. ***
C
      ierr=0
      CALL READMEM(filmem,title,CELL,pop,iptyp,jph,jpk,jpl,
     $  nsg,nset,NPIX,mpix,totale,ramda,mref,nref,nref1,nref2,
     $  IDX,FOBS,SIGF,GOBS,SIGG,NGRP,PMUL,PMULS,ierr,igsw)
      if (ierr.ne.0) goto 201
C
      CALL RDSPG(nsg,nset,aspg,nsym,ncen,nsub,IROT,RTRS,RSUB)
C
      CALL PRECAL(jph,jpk,jpl,pi,rad,ca,cb,cg,sa,sb,sg,
     $   da,db,dg,dv,sv,as,bs,cs,ga,gb,gg,rh,rk,rl,av,
     $   rdint,totale,ff1,ff2,CELL,NPIX)

      CALL POCOR(iptyp,pop,PROF,nref,mref,IDX,
     $	               as,bs,cs,ga,gb,gg,rh,rk,rl,pi)
C
C*** INITIALIZE DATA & OUTPUT INITIAL STATE. ***
C
  210 WRITE(6,3130)
      IF (IPRIOR.EQ.0) THEN
         CALL CMKASYM(0,masym,nasym,mpix,NPIX,
     $     IROT,RTRS,RSUB,nsym,ncen,nsub,
     $     RDEN,rtasym,RASYM,rdint, IASYM,RMIC,
     $     RMIS,
     $     IDX,mref,nref,IDE,pi)
      ELSE
	 CALL READDEN(filden,masym,nasym,IASYM,RDEN)
         CALL CMKASYM(1,masym,nasym,mpix,NPIX,
     $     IROT,RTRS,RSUB,nsym,ncen,nsub,
     $     RDEN,rtasym,RASYM,rdint, IASYM,RMIC,
     $     RMIS,
     $     IDX,mref,nref,IDE,pi)
      END IF
C     print *,' nsym,nsub,ncen'
C     print *,nsym,nsub,ncen,(npix(i),i=1,3)
C     write(*,'(12i5)') (((irot(i,j,k),I=1,3),j=1,3),
C    $  (nint(npix(i)*rtrs(i,k)),i=1,3),
C    $    k=1,nsym)
C     write(*,'(3i5)') ((nint(npix(i)*rsub(i,k)),i=1,3),k=1,nsub)
C     read(*,*) idummi
C
C     ***** do precalculations for maxent *****
C
      NHIST=0
      itcycl=1
      delta=-1.0D-9
      ngsw=0
C     *** if only Fobs are given: ngsw=1
C     *** if Gobs are given: in the first cycle, ngsw=-1 and the Gobs are 
C     *** not considered in the maxent-process. This is because Gcal is 
C     *** zero for flat starting-density, and there would result a `divided
C     *** by zero' error in the MAXENT routine.
C     *** After the first cycle, ngsw is set 0 in this case.
      IF (iprior.EQ.0) ngsw=-1
      IF (igsw.NE.0) ngsw=1
C     *** rden --> rdeo; calculate and memorize F_calc ***
      CALL RENEW(masym,nasym,RDEO,RDEN)
      CALL CSTRCF(masym,nasym,RDEN,RMIC
     $   ,RMIS
     $   ,RASYM,mref,nref,FCAL,ff1)
C     *** memorize FCAL
      DO 42 iref=1,NREF
	 FCALO(1,iref)=FCAL(1,iref)
	 FCALO(2,iref)=FCAL(2,iref)
42    CONTINUE
      NN=nref1
      IF (NGSW.eq.0) THEN
         NN=NN+nref2
      ENDIF
      IF (NGSW.le.0) THEN
         CALL CSTRCG( mref,nref1,nref2,FCAL,GCAL,
     $      PROF,NGRP,PMUL,PMULS)
C        *** memorize GCAL
         DO 43 iref=1,nref2
 	   GCALO(iref)=GCAL(iref)
	   GCALO(iref)=GCAL(iref)
43       CONTINUE
      ENDIF
      TNREF=DFLOAT(NN)
C     *** calculate constraint values ***
      CALL CCALCON(ngsw,conf,cong,wrfact,rfact,connew,tnref,
     $   mref,nref1,nref2,FCAL,PROF,FOBS,SIGF,GCAL,GOBS,SIGG )
      conold=connew
C     *** calculate entropy-function values ***
      CALL CALFN(masym,nasym,entj,ents,rtasym,RDEN,RDEO,RASYM,
     $   ff2,qfunc,ramda,connew)
      REWIND(30)
      DO 60 I=1,2
         NN=6
         IF (I.EQ.2) NN=30
         WRITE(NN,3020)
     $   TITLE,FILMEM,FILDEN,FILDOC,CELL,ASPG,TOTALE,POP,IPTYP,
     $   JPH,JPK,JPL,NASYM,NPIX(1)*NPIX(2)*NPIX(3),(NPIX(J),J=1,3),
     $   nref1,nref2,APRIOR(IPRIOR+1),AGSW(IGSW+1),RAMDA
         WRITE(NN,3030) connew,conf,cong,delta,rfact,wrfact
         WRITE(NN,3040) ENTS,ENTJ,QFUNC
   60 CONTINUE
C
C     ************************************************************
C     *** MAXENT PROCESS ***
C     ************************************************************
C
C     ***** start of loop over all cycles of calculation *********
C
  220 WRITE(6,3560)
      READ(5,2020,ERR=220) NCYCL
      IF (NCYCL.LT.-100) then
C       *** start from the beginning ***
        GO TO 210
      elseif (ncycl.lt.0) then
C       *** input different lambda/alpha ***
        CALL HISTRY(itcycl,delta,alpha,ramda,nhist,
     $                  mhist,FHIST,IHIST)
        goto 220
      elseif (ncycl.eq.0) then
C       *** stop calculations ***
        print *,
     $    'Do you really want to stop the calculation ? (0=yes)'
        iii=0
        read(5,2020) iii
        if (iii.ne.0) goto 220
C       *** stop and save density in file: ***
        goto 260 
      elseif (ncycl.gt. 0) then
C       *** normal case: positive number of cycles given ***
	icycl=1
        ibadruns=0
        igoodcyc=0
      endif
C
C     ***** start of loop over NCYCL cycles *********
C
240   if (ibadruns.gt.13) then
C        *** exit loop: save and display last sensible results ***
	 print *,'-------------------------------------------'
	 print *,'Exit with saving of data: divergence error!'
	 print *,'-------------------------------------------'
	 itcycl=igoodcyc
	 connew=conold
	 conf=confold
         cong=congold
	 delta=deltaold
	 rfact=rfacold
	 wrfact=wrfacold
C        *** take old density as result ***
	 do 245,ia=1,nasym
	    rden(ia)=rdeo(ia)
245      continue
         CALL CSTRCF(masym,nasym,RDEN,RMIC
     $      ,RMIS
     $      ,RASYM,mref,nref,FCAL,ff1)
	 if (ngsw.eq.0) CALL CSTRCG(mref,nref1,nref2,FCAL,GCAL,
     $                     PROF,NGRP,PMUL,PMULS)
	 goto 260
      endif
C
      CALL CMAXENT(ngsw,mref,nref1,nref2,masym,nasym,rtasym
     $  ,FOBS,FCAL,PROF,SIGF,NGRP,GOBS,GCAL,SIGG,RMIC
     $  ,RMIS
     $  ,tnref,ramda,RDEO,RDEN,RHELP1,RHELP2,RHELP4,RHELP5,RHELP6)
C
      CALL NORMAL(masym,nasym,RDEN,RASYM,ff2,rtasym)
      CALL CSTRCF(masym,nasym,RDEN,RMIC
     $  ,RMIS
     $  ,RASYM,mref,nref,FCAL,ff1)
      if (ngsw.le.0) CALL CSTRCG( mref,nref1,nref2,FCAL,GCAL,
     $                            PROF,NGRP,PMUL,PMULS)
      CALL CCALCON(ngsw,conf,cong,wrfact,rfact,connew,tnref,
     $  mref,nref1,nref2,FCAL,PROF,FOBS,SIGF,GCAL,GOBS,SIGG )
      delta=connew-conold
C
      WRITE(6,3060) itcycl
      WRITE(6,3030) connew,conf,cong,delta,RFACT,WRFACT
      IF (delta.GE.0.0) then
C        *** divergence error: choose new ramda and restart calc. ***
	 ibadruns=ibadruns+1
	 ramda=ramda/2
	 write(*,'(a,i3,a,f13.8)') 'Bad cycles number:',ibadruns,
     $    '   New ramda=',ramda
C        *** call back F_cal, G_cal of last successful cycle ***
C        *** nref=nref1 + (No. of reflections in groups) ***
         DO 242 iref=1,NREF
  	   FCAL(1,iref)=FCALO(1,iref)
	   FCAL(2,iref)=FCALO(2,iref)
242      CONTINUE
         IF (NGSW.le.0) THEN
           DO 243 iref=1,nref2
             GCAL(iref)=GCALO(iref)
	     GCAL(iref)=GCALO(iref)
243        CONTINUE
         ENDIF
C        *** restart calculation with changed ramda ***
         GO TO 240
      ENDIF 
C     *** this cyle was successful: delta < 0 ***
C     *** new density rden --> rdeo ***
      CALL RENEW(masym,nasym,rdeo,rden)
C     *** increase ramda a bit ***
      ramda=ramda*1.02
C     *** memorize a lot of this cycle ***
      ibadruns=0
      igoodcyc=itcycl
      confold=conf
      congold=cong
      deltaold=delta
      rfacold=rfact
      wrfacold=wrfact
      DO 252 iref=1,nref
         FCALO(1,iref)=FCAL(1,iref)
	 FCALO(2,iref)=FCAL(2,iref)
252   CONTINUE
      IF (ngsw.le.0) THEN
         DO 253 iref=1,nref2
 	   GCALO(iref)=GCAL(iref)
	   GCALO(iref)=GCAL(iref)
253      CONTINUE
      ENDIF
C     *** increase cycle counters ***
      itcycl=itcycl+1
      icycl=icycl+1
C     *** SAVE DENSITY DATA after every 50th cycle ***
      if (mod(itcycl-1,50).eq.0) then
         print *,'Automatically saving density data ...'
	 CALL SAVEDEN(filden,
     $      masym,nasym,title,IASYM,RDEN,CELL,NPIX
     $      ,nsym,ncen,nsub,IROT,RTRS,RSUB )
      endif
C
      IF (connew.LE.ALPHA) then
C        *** calculation finished ***  
	 GO TO 260
      endif
      conold=connew
      IF (ngsw.EQ.-1) then
	 ngsw=0
	 nn=nref1+nref2
	 tnref=float(nn)
      ENDIF
      IF (icycl.LE.ncycl) GO TO 240
C
C     ***** end of loop over NCYCL cycles *********
C
C     CALL DSPTIM(6,itcycl)
      GO TO 220
C
C     ***** end of loop over all cycles of calculation *********
C     ==========================================================
C
C     **********  DISPLAY & OUTPUT ANALYSIS RESULT.  ***********
C
260   write(6,3000)
C
C*** SAVE DENSITY DATA. ***
C
      CALL SAVEDEN(filden,masym,nasym,title,IASYM,RDEN,CELL,NPIX
     $                ,nsym,ncen,nsub,IROT,RTRS,RSUB )
C
C*** CALCULATE ENTROPY-FUNCTIONS
      CALL CALFN(masym,nasym,entj,ents,rtasym,RDEN,RDEO,RASYM,
     $   ff2,qfunc,ramda,connew)
C
C*** DISPLAY INTO STANDARD OUTPUT ***
      CALL dspresult(6,
     $  connew,conf,cong,delta,rfact,wrfact,ents,entj,qfunc,
     $  IHIST,FHIST,mhist,nhist,iptyp,
     $  mref,nref1,nref2,FOBS,FCAL,PROF,NGRP,GOBS,GCAL,IDX)
C
C*** DISPLAY INTO DOCUMENT FILE ***
      OPEN(30,FILE=FILDOC,FORM='FORMATTED',STATUS='UNKNOWN',
     $     ACCESS='SEQUENTIAL')
      CALL dspresult(30,
     $  connew,conf,cong,delta,rfact,wrfact,ents,entj,qfunc,
     $  IHIST,FHIST,mhist,nhist,iptyp,
     $  mref,nref1,nref2,FOBS,FCAL,PROF,NGRP,GOBS,GCAL,IDX)
      CLOSE(30)
C
C*** DISPLAY FIRST DENSITY VAUES
      print *,'First 10 values of density:'
      write(*,1090) ((iasym(j,i),j=1,3),RDEN(i),i=1,10)
      print *
C
C*** END OF PROGRAM ***
C
      STOP
      END
